@@ -60,18 +60,18 @@ function (x,
           cross       = 0,
           probability = FALSE,
           fitted      = TRUE,
-          seed        = 1L,
           ...,
           subset,
           na.action = na.omit)
 {
+    yorig <- y
     if(inherits(x, "Matrix")) {
-        library("SparseM")
-        library("Matrix")
+        loadNamespace("SparseM")
+        loadNamespace("Matrix")
         x <- as(x, "matrix.csr")
     }
     if(inherits(x, "simple_triplet_matrix")) {
-        library("SparseM")
+        loadNamespace("SparseM")
         ind <- order(x$i, x$j)
         x <- new("matrix.csr",
                  ra = x$v[ind],
@@ -80,7 +80,7 @@ function (x,
                  dimension = c(x$nrow, x$ncol))
     }
     if (sparse <- inherits(x, "matrix.csr"))
-        library("SparseM")
+        loadNamespace("SparseM")
 
     ## NULL parameters?
     if(is.null(degree)) stop(sQuote("degree"), " must not be NULL!")
@@ -91,7 +91,7 @@ function (x,
     if(is.null(epsilon)) stop(sQuote("epsilon"), " must not be NULL!")
     if(is.null(tolerance)) stop(sQuote("tolerance"), " must not be NULL!")
 
-    xhold   <- if (fitted) x else NA
+    xhold   <- if (fitted) x else NULL
     x.scale <- y.scale <- NULL
     formula <- inherits(x, "svm.formula")
 
@@ -128,13 +128,18 @@ function (x,
 
         ## subsetting and na-handling for matrices
         if (!formula) {
-            if (!missing(subset)) x <- x[subset,]
+            if (!missing(subset)) {
+                x <- x[subset,]
+                y <- y[subset]
+                if (!is.null(xhold))
+                    xhold <- as.matrix(xhold)[subset,]
+            }
             if (is.null(y))
                 x <- na.action(x)
             else {
                 df <- na.action(data.frame(y, x))
                 y <- df[,1]
-                x <- as.matrix(df[,-1])
+                x <- as.matrix(df[,-1], rownames.force = TRUE)
                 nac <-
                     attr(x, "na.action") <-
                         attr(y, "na.action") <-
@@ -160,6 +165,7 @@ function (x,
                 x[,scale] <- xtmp
                 x.scale <- attributes(xtmp)[c("scaled:center","scaled:scale")]
                 if (is.numeric(y) && (type > 2)) {
+                    yorig <- y
                     y <- scale(y)
                     y.scale <- attributes(y)[c("scaled:center","scaled:scale")]
                     y <- as.vector(y)
@@ -173,7 +179,8 @@ function (x,
     if (cross > nr)
         stop(sQuote("cross"), " cannot exceed the number of observations!")
 
-    if (!is.vector(y) && !is.factor (y) && type != 2)
+    if (!is.vector(`attributes<-`(y, NULL))
+        && !is.factor (y) && type != 2)
         stop("y must be a vector or a factor.")
     if (type != 2 && length(y) != nr)
         stop("x and y don't match.")
@@ -236,7 +243,6 @@ function (x,
     if (is.null(cross)) stop("cross argument must not be NULL!")
     if (is.null(sparse)) stop("sparse argument must not be NULL!")
     if (is.null(probability)) stop("probability argument must not be NULL!")
-    if (is.null(seed)) stop("seed argument must not be NULL!")
 
     cret <- .C ("svmtrain",
                 ## data
@@ -265,7 +271,6 @@ function (x,
                 as.integer (cross),
                 as.integer (sparse),
                 as.integer (probability),
-                as.integer (seed),
 
                 ## results
                 nclasses = integer  (1),
@@ -310,7 +315,7 @@ function (x,
                  levels   = lev,
                  tot.nSV  = cret$nr, #total number of sv
                  nSV      = cret$nSV[1:cret$nclasses], #number of SV in diff. classes
-                 labels   = cret$label[1:cret$nclasses], #labels of the SVs.
+                 labels   = cret$labels[1:cret$nclasses], #labels of the SVs.
                  SV       = if (sparse) SparseM::t(SparseM::t(x[cret$index,]))
                  else t(t(x[cret$index,])), #copy of SV
                  index    = cret$index,  #indexes of sv in x
@@ -350,7 +355,7 @@ function (x,
                                         decision.values = TRUE))
         ret$decision.values <- attr(ret$fitted, "decision.values")
         attr(ret$fitted, "decision.values") <- NULL
-        if (type > 1) ret$residuals <- y - ret$fitted
+        if (type > 1) ret$residuals <- yorig - ret$fitted
     }
 
     ret
@@ -371,12 +376,12 @@ function (object, newdata,
 
 
     if(inherits(newdata, "Matrix")) {
-        library("SparseM")
-        library("Matrix")
+        loadNamespace("SparseM")
+        loadNamespace("Matrix")
         newdata <- as(newdata, "matrix.csr")
     }
     if(inherits(newdata, "simple_triplet_matrix")) {
-       library("SparseM")
+       loadNamespace("SparseM")
        ind <- order(newdata$i, newdata$j)
        newdata <- new("matrix.csr",
                       ra = newdata$v[ind],
@@ -387,7 +392,7 @@ function (object, newdata,
 
     sparse <- inherits(newdata, "matrix.csr")
     if (object$sparse || sparse)
-        library("SparseM")
+        loadNamespace("SparseM")
 
     act <- NULL
     if ((is.vector(newdata) && is.atomic(newdata)))
@@ -403,10 +408,10 @@ function (object, newdata,
         if (inherits(object, "svm.formula")) {
             if(is.null(colnames(newdata)))
                 colnames(newdata) <- colnames(object$SV)
-            newdata <- na.action(newdata)
-            act <- attr(newdata, "na.action")
             newdata <- model.matrix(delete.response(terms(object)),
                                     as.data.frame(newdata))
+            newdata <- na.action(newdata)
+            act <- attr(newdata, "na.action")
         } else {
             newdata <- na.action(as.matrix(newdata))
             act <- attr(newdata, "na.action")
@@ -438,8 +443,8 @@ function (object, newdata,
                as.double  (as.vector(object$coefs)),
                as.double  (object$rho),
                as.integer (object$compprob),
-               as.double  (object$probA),
-               as.double  (object$probB),
+               as.double  (if (object$compprob) object$probA else 0),
+               as.double  (if (object$compprob) object$probB else 0),
                as.integer (object$nclasses),
                as.integer (object$tot.nSV),
                as.integer (object$labels),
@@ -496,13 +501,17 @@ function (object, newdata,
                       )
     }
 
-    if (probability && object$type < 2)
-        attr(ret2, "probabilities") <-
-            napredict(act,
-                      matrix(ret$prob, nrow = nrow(newdata), byrow = TRUE,
-                             dimnames = list(rowns, object$levels[object$labels])
-                             )
-                      )
+    if (probability && object$type < 2) {
+        if (!object$compprob)
+            warning("SVM has not been trained using `probability = TRUE`, probabilities not available for predictions.")
+        else
+            attr(ret2, "probabilities") <-
+                napredict(act,
+                          matrix(ret$prob, nrow = nrow(newdata), byrow = TRUE,
+                                 dimnames = list(rowns, object$levels[object$labels])
+                                 )
+                          )
+    }
 
     ret2
 }
@@ -674,8 +683,9 @@ function (object, svm.file="Rdata.svm", scale.file = "Rdata.scale",
                as.integer (if (object$sparse) object$SV@ja else 0),
                as.double  (as.vector(object$coefs)),
                as.double  (object$rho),
-               as.double  (object$probA),
-               as.double  (object$probB),
+               as.integer (object$compprob),
+               as.double  (if (object$compprob) object$probA else 0),
+               as.double  (if (object$compprob) object$probB else 0),
                as.integer (object$nclasses),
                as.integer (object$tot.nSV),
                as.integer (object$labels),
